@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2010 Redpill Linpro AS
+ * Copyright (c) 2006-2011 Varnish Software AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -32,9 +32,6 @@
 
 #include "config.h"
 
-#include "svnid.h"
-SVNID("$Id: cache_backend.c 5434 2010-10-19 15:01:04Z tfheen $")
-
 #include <limits.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -44,52 +41,57 @@ SVNID("$Id: cache_backend.c 5434 2010-10-19 15:01:04Z tfheen $")
 
 #include <sys/socket.h>
 
-#include "shmlog.h"
 #include "cache.h"
 #include "cache_backend.h"
 #include "vrt.h"
 
-/*
- * List of cached vbe_conns, used if enabled in params/heritage
+/*--------------------------------------------------------------------
+ * The "simple" director really isn't, since thats where all the actual
+ * connections happen.  Nontheless, pretend it is simple by sequestering
+ * the directoricity of it under this line.
  */
-static VTAILQ_HEAD(,vbe_conn) vbe_conns = VTAILQ_HEAD_INITIALIZER(vbe_conns);
+
+struct vdi_simple {
+	unsigned		magic;
+#define VDI_SIMPLE_MAGIC	0x476d25b7
+	struct director		dir;
+	struct backend		*backend;
+	const struct vrt_backend *vrt;
+};
 
 /*--------------------------------------------------------------------
  * Create default Host: header for backend request
  */
 void
-VBE_AddHostHeader(const struct sess *sp)
+VDI_AddHostHeader(const struct sess *sp)
 {
 
 	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
 	CHECK_OBJ_NOTNULL(sp->wrk->bereq, HTTP_MAGIC);
-	CHECK_OBJ_NOTNULL(sp->vbe, VBE_CONN_MAGIC);
-	CHECK_OBJ_NOTNULL(sp->vbe->backend, BACKEND_MAGIC);
+	CHECK_OBJ_NOTNULL(sp->vbc, VBC_MAGIC);
+	CHECK_OBJ_NOTNULL(sp->vbc->vdis, VDI_SIMPLE_MAGIC);
 	http_PrintfHeader(sp->wrk, sp->fd, sp->wrk->bereq,
-	    "Host: %s", sp->vbe->backend->hosthdr);
+	    "Host: %s", sp->vbc->vdis->vrt->hosthdr);
 }
 
+/*--------------------------------------------------------------------*/
+
 /* Private interface from backend_cfg.c */
 void
-VBE_ReleaseConn(struct vbe_conn *vc)
+VBE_ReleaseConn(struct vbc *vc)
 {
 
-	CHECK_OBJ_NOTNULL(vc, VBE_CONN_MAGIC);
+	CHECK_OBJ_NOTNULL(vc, VBC_MAGIC);
 	assert(vc->backend == NULL);
 	assert(vc->fd < 0);
 
+	vc->addr = NULL;
+	vc->addrlen = 0;
 	vc->recycled = 0;
-	if (params->cache_vbe_conns) {
-		Lck_Lock(&VBE_mtx);
-		VTAILQ_INSERT_HEAD(&vbe_conns, vc, list);
-		VSL_stats->backend_unused++;
-		Lck_Unlock(&VBE_mtx);
-	} else {
-		Lck_Lock(&VBE_mtx);
-		VSL_stats->n_vbe_conn--;
-		Lck_Unlock(&VBE_mtx);
-		free(vc);
-	}
+	Lck_Lock(&VBE_mtx);
+	VSC_C_main->n_vbc--;
+	Lck_Unlock(&VBE_mtx);
+	FREE_OBJ(vc);
 }
 
 #define FIND_TMO(tmx, dst, sp, be)		\
@@ -101,7 +103,6 @@ VBE_ReleaseConn(struct vbe_conn *vc)
 			dst = params->tmx;	\
 	} while (0)
 
-
 /*--------------------------------------------------------------------
  * Attempt to connect to a given addrinfo entry.
  *
@@ -112,48 +113,49 @@ VBE_ReleaseConn(struct vbe_conn *vc)
  */
 
 static int
-vbe_TryConnect(const struct sess *sp, int pf, const struct sockaddr *sa,
-    socklen_t salen, const struct backend *bp)
+vbe_TryConnect(const struct sess *sp, int pf, const struct sockaddr_storage *sa,
+    socklen_t salen, const struct vdi_simple *vs)
 {
 	int s, i, tmo;
 	double tmod;
-	char abuf1[TCP_ADDRBUFSIZE], abuf2[TCP_ADDRBUFSIZE];
-	char pbuf1[TCP_PORTBUFSIZE], pbuf2[TCP_PORTBUFSIZE];
+	char abuf1[VTCP_ADDRBUFSIZE], abuf2[VTCP_ADDRBUFSIZE];
+	char pbuf1[VTCP_PORTBUFSIZE], pbuf2[VTCP_PORTBUFSIZE];
 
 	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
+	CHECK_OBJ_NOTNULL(vs, VDI_SIMPLE_MAGIC);
 
 	s = socket(pf, SOCK_STREAM, 0);
 	if (s < 0)
 		return (s);
 
-	FIND_TMO(connect_timeout, tmod, sp, bp);
+	FIND_TMO(connect_timeout, tmod, sp, vs->vrt);
 
 	tmo = (int)(tmod * 1000.0);
 
-	if (tmo > 0)
-		i = TCP_connect(s, sa, salen, tmo);
-	else
-		i = connect(s, sa, salen);
+	i = VTCP_connect(s, sa, salen, tmo);
 
 	if (i != 0) {
 		AZ(close(s));
 		return (-1);
 	}
 
-	TCP_myname(s, abuf1, sizeof abuf1, pbuf1, sizeof pbuf1);
-	TCP_name(sa, salen, abuf2, sizeof abuf2, pbuf2, sizeof pbuf2);
+	VTCP_myname(s, abuf1, sizeof abuf1, pbuf1, sizeof pbuf1);
+	VTCP_name(sa, salen, abuf2, sizeof abuf2, pbuf2, sizeof pbuf2);
 	WSL(sp->wrk, SLT_BackendOpen, s, "%s %s %s %s %s",
-	    bp->vcl_name, abuf1, pbuf1, abuf2, pbuf2);
+	    vs->backend->vcl_name, abuf1, pbuf1, abuf2, pbuf2);
 
 	return (s);
 }
 
 /*--------------------------------------------------------------------*/
 
-static int
-bes_conn_try(const struct sess *sp, struct backend *bp)
+static void
+bes_conn_try(const struct sess *sp, struct vbc *vc, const struct vdi_simple *vs)
 {
 	int s;
+	struct backend *bp = vs->backend;
+
+	CHECK_OBJ_NOTNULL(vs, VDI_SIMPLE_MAGIC);
 
 	Lck_Lock(&bp->mtx);
 	bp->refcount++;
@@ -165,20 +167,31 @@ bes_conn_try(const struct sess *sp, struct backend *bp)
 
 	/* release lock during stuff that can take a long time */
 
-	if (params->prefer_ipv6 && bp->ipv6 != NULL)
-		s = vbe_TryConnect(sp, PF_INET6, bp->ipv6, bp->ipv6len, bp);
-	if (s == -1 && bp->ipv4 != NULL)
-		s = vbe_TryConnect(sp, PF_INET, bp->ipv4, bp->ipv4len, bp);
-	if (s == -1 && !params->prefer_ipv6 && bp->ipv6 != NULL)
-		s = vbe_TryConnect(sp, PF_INET6, bp->ipv6, bp->ipv6len, bp);
+	if (params->prefer_ipv6 && bp->ipv6 != NULL) {
+		s = vbe_TryConnect(sp, PF_INET6, bp->ipv6, bp->ipv6len, vs);
+		vc->addr = bp->ipv6;
+		vc->addrlen = bp->ipv6len;
+	}
+	if (s == -1 && bp->ipv4 != NULL) {
+		s = vbe_TryConnect(sp, PF_INET, bp->ipv4, bp->ipv4len, vs);
+		vc->addr = bp->ipv4;
+		vc->addrlen = bp->ipv4len;
+	}
+	if (s == -1 && !params->prefer_ipv6 && bp->ipv6 != NULL) {
+		s = vbe_TryConnect(sp, PF_INET6, bp->ipv6, bp->ipv6len, vs);
+		vc->addr = bp->ipv6;
+		vc->addrlen = bp->ipv6len;
+	}
 
+	vc->fd = s;
 	if (s < 0) {
 		Lck_Lock(&bp->mtx);
 		bp->n_conn--;
 		bp->refcount--;		/* Only keep ref on success */
 		Lck_Unlock(&bp->mtx);
+		vc->addr = NULL;
+		vc->addrlen = 0;
 	}
-	return (s);
 }
 
 /*--------------------------------------------------------------------
@@ -199,43 +212,29 @@ vbe_CheckFd(int fd)
 }
 
 /*--------------------------------------------------------------------
- * Manage a pool of vbe_conn structures.
+ * Manage a pool of vbc structures.
  * XXX: as an experiment, make this caching controled by a parameter
  * XXX: so we can see if it has any effect.
  */
 
-static struct vbe_conn *
+static struct vbc *
 vbe_NewConn(void)
 {
-	struct vbe_conn *vc;
+	struct vbc *vc;
 
-	vc = VTAILQ_FIRST(&vbe_conns);
-	if (vc != NULL) {
-		Lck_Lock(&VBE_mtx);
-		vc = VTAILQ_FIRST(&vbe_conns);
-		if (vc != NULL) {
-			VSL_stats->backend_unused--;
-			VTAILQ_REMOVE(&vbe_conns, vc, list);
-		}
-		Lck_Unlock(&VBE_mtx);
-	}
-	if (vc != NULL)
-		return (vc);
-	vc = calloc(sizeof *vc, 1);
+	ALLOC_OBJ(vc, VBC_MAGIC);
 	XXXAN(vc);
-	vc->magic = VBE_CONN_MAGIC;
 	vc->fd = -1;
 	Lck_Lock(&VBE_mtx);
-	VSL_stats->n_vbe_conn++;
+	VSC_C_main->n_vbc++;
 	Lck_Unlock(&VBE_mtx);
 	return (vc);
 }
 
-
 /*--------------------------------------------------------------------
  * It evaluates if a backend is healthy _for_a_specific_object_.
- * That means that it relies on sp->objhead. This is mainly for saint-mode,
- * but also takes backend->healthy into account. If
+ * That means that it relies on sp->objcore->objhead. This is mainly for
+ * saint-mode, but also takes backend->healthy into account. If
  * params->saintmode_threshold is 0, this is basically just a test of
  * backend->healthy.
  *
@@ -244,14 +243,20 @@ vbe_NewConn(void)
  */
 
 static unsigned int
-vbe_Healthy(double now, uintptr_t target, struct backend *backend)
+vbe_Healthy(const struct vdi_simple *vs, const struct sess *sp)
 {
 	struct trouble *tr;
 	struct trouble *tr2;
 	struct trouble *old;
 	unsigned i = 0, retval;
 	unsigned int threshold;
+	struct backend *backend;
+	uintptr_t target;
+	double now;
 
+	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
+	CHECK_OBJ_NOTNULL(vs, VDI_SIMPLE_MAGIC);
+	backend = vs->backend;
 	CHECK_OBJ_NOTNULL(backend, BACKEND_MAGIC);
 
 	if (!backend->healthy)
@@ -260,21 +265,21 @@ vbe_Healthy(double now, uintptr_t target, struct backend *backend)
 	/* VRT/VCC sets threshold to UINT_MAX to mark that it's not
 	 * specified by VCL (thus use param).
 	 */
-	if (backend->saintmode_threshold == UINT_MAX)
+	if (vs->vrt->saintmode_threshold == UINT_MAX)
 		threshold = params->saintmode_threshold;
 	else
-		threshold = backend->saintmode_threshold;
+		threshold = vs->vrt->saintmode_threshold;
 
 	/* Saintmode is disabled */
 	if (threshold == 0)
 		return (1);
 
-	/* No need to test if we don't have an object head to test against.
-	 * FIXME: Should check the magic too, but probably not assert?
-	 */
-	if (target == 0)
+	if (sp->objcore == NULL)
 		return (1);
 
+	now = sp->t_req;
+	target = (uintptr_t)(sp->objcore->objhead);
+
 	old = NULL;
 	retval = 1;
 	Lck_Lock(&backend->mtx);
@@ -314,15 +319,18 @@ vbe_Healthy(double now, uintptr_t target, struct backend *backend)
  * Get a connection to a particular backend.
  */
 
-static struct vbe_conn *
-vbe_GetVbe(struct sess *sp, struct backend *bp)
+static struct vbc *
+vbe_GetVbe(const struct sess *sp, struct vdi_simple *vs)
 {
-	struct vbe_conn *vc;
+	struct vbc *vc;
+	struct backend *bp;
 
 	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
+	CHECK_OBJ_NOTNULL(vs, VDI_SIMPLE_MAGIC);
+	bp = vs->backend;
 	CHECK_OBJ_NOTNULL(bp, BACKEND_MAGIC);
 
-	/* first look for vbe_conn's we can recycle */
+	/* first look for vbc's we can recycle */
 	while (1) {
 		Lck_Lock(&bp->mtx);
 		vc = VTAILQ_FIRST(&bp->connlist);
@@ -330,6 +338,7 @@ vbe_GetVbe(struct sess *sp, struct backend *bp)
 			bp->refcount++;
 			assert(vc->backend == bp);
 			assert(vc->fd >= 0);
+			AN(vc->addr);
 			VTAILQ_REMOVE(&bp->connlist, vc, list);
 		}
 		Lck_Unlock(&bp->mtx);
@@ -337,158 +346,61 @@ vbe_GetVbe(struct sess *sp, struct backend *bp)
 			break;
 		if (vbe_CheckFd(vc->fd)) {
 			/* XXX locking of stats */
-			VSL_stats->backend_reuse += 1;
+			VSC_C_main->backend_reuse += 1;
 			WSP(sp, SLT_Backend, "%d %s %s",
 			    vc->fd, sp->director->vcl_name, bp->vcl_name);
+			vc->vdis = vs;
 			vc->recycled = 1;
 			return (vc);
 		}
-		VSL_stats->backend_toolate++;
+		VSC_C_main->backend_toolate++;
 		WSL(sp->wrk, SLT_BackendClose, vc->fd, "%s", bp->vcl_name);
 
 		/* Checkpoint log to flush all info related to this connection
 		   before the OS reuses the FD */
 		WSL_Flush(sp->wrk, 0);
 
-		TCP_close(&vc->fd);
+		VTCP_close(&vc->fd);
 		VBE_DropRefConn(bp);
 		vc->backend = NULL;
 		VBE_ReleaseConn(vc);
 	}
 
-	if (!vbe_Healthy(sp->t_req, (uintptr_t)sp->objhead, bp)) {
-		VSL_stats->backend_unhealthy++;
+	if (!vbe_Healthy(vs, sp)) {
+		VSC_C_main->backend_unhealthy++;
 		return (NULL);
 	}
 
-	if (bp->max_conn > 0 && bp->n_conn >= bp->max_conn) {
-		VSL_stats->backend_busy++;
+	if (vs->vrt->max_connections > 0 &&
+	    bp->n_conn >= vs->vrt->max_connections) {
+		VSC_C_main->backend_busy++;
 		return (NULL);
 	}
 
 	vc = vbe_NewConn();
 	assert(vc->fd == -1);
 	AZ(vc->backend);
-	vc->fd = bes_conn_try(sp, bp);
+	bes_conn_try(sp, vc, vs);
 	if (vc->fd < 0) {
 		VBE_ReleaseConn(vc);
-		VSL_stats->backend_fail++;
+		VSC_C_main->backend_fail++;
 		return (NULL);
 	}
 	vc->backend = bp;
-	VSL_stats->backend_conn++;
+	VSC_C_main->backend_conn++;
 	WSP(sp, SLT_Backend, "%d %s %s",
 	    vc->fd, sp->director->vcl_name, bp->vcl_name);
+	vc->vdis = vs;
 	return (vc);
 }
 
-/* Close a connection ------------------------------------------------*/
-
-void
-VBE_CloseFd(struct sess *sp)
-{
-	struct backend *bp;
-
-	CHECK_OBJ_NOTNULL(sp->vbe, VBE_CONN_MAGIC);
-	CHECK_OBJ_NOTNULL(sp->vbe->backend, BACKEND_MAGIC);
-	assert(sp->vbe->fd >= 0);
-
-	bp = sp->vbe->backend;
-
-	WSL(sp->wrk, SLT_BackendClose, sp->vbe->fd, "%s", bp->vcl_name);
-
-	/* Checkpoint log to flush all info related to this connection
-	   before the OS reuses the FD */
-	WSL_Flush(sp->wrk, 0);
-
-	TCP_close(&sp->vbe->fd);
-	VBE_DropRefConn(bp);
-	sp->vbe->backend = NULL;
-	VBE_ReleaseConn(sp->vbe);
-	sp->vbe = NULL;
-}
-
-/* Recycle a connection ----------------------------------------------*/
-
-void
-VBE_RecycleFd(struct sess *sp)
-{
-	struct backend *bp;
-
-	CHECK_OBJ_NOTNULL(sp->vbe, VBE_CONN_MAGIC);
-	CHECK_OBJ_NOTNULL(sp->vbe->backend, BACKEND_MAGIC);
-	assert(sp->vbe->fd >= 0);
-
-	bp = sp->vbe->backend;
-
-	WSL(sp->wrk, SLT_BackendReuse, sp->vbe->fd, "%s", bp->vcl_name);
-	/*
-	 * Flush the shmlog, so that another session reusing this backend
-	 * will log chronologically later than our use of it.
-	 */
-	WSL_Flush(sp->wrk, 0);
-	Lck_Lock(&bp->mtx);
-	VSL_stats->backend_recycle++;
-	VTAILQ_INSERT_HEAD(&bp->connlist, sp->vbe, list);
-	sp->vbe = NULL;
-	VBE_DropRefLocked(bp);
-}
-
-/* Get a connection --------------------------------------------------*/
-
-struct vbe_conn *
-VBE_GetFd(const struct director *d, struct sess *sp)
-{
-
-	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
-	if (d == NULL)
-		d = sp->director;
-	CHECK_OBJ_NOTNULL(d, DIRECTOR_MAGIC);
-	return (d->getfd(d, sp));
-}
-
-/* Check health ------------------------------------------------------
- *
- * The target is really an objhead pointer, but since it can not be
- * dereferenced during health-checks, we pass it as uintptr_t, which
- * hopefully will make people investigate, before mucking about with it.
- */
-
-int
-VBE_Healthy_sp(const struct sess *sp, const struct director *d)
-{
-
-	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
-	CHECK_OBJ_NOTNULL(d, DIRECTOR_MAGIC);
-	return (d->healthy(sp->t_req, d, (uintptr_t)sp->objhead));
-}
-
-int
-VBE_Healthy(double now, const struct director *d, uintptr_t target)
-{
-
-	CHECK_OBJ_NOTNULL(d, DIRECTOR_MAGIC);
-	return (d->healthy(now, d, target));
-}
-
 /*--------------------------------------------------------------------
- * The "simple" director really isn't, since thats where all the actual
- * connections happen.  Nontheless, pretend it is simple by sequestering
- * the directoricity of it under this line.
- */
-
-struct vdi_simple {
-	unsigned		magic;
-#define VDI_SIMPLE_MAGIC	0x476d25b7
-	struct director		dir;
-	struct backend		*backend;
-};
-
-/* Returns the backend if and only if the this is a simple director.
+ * Returns the backend if and only if the this is a simple director.
  * XXX: Needs a better name and possibly needs a better general approach.
  * XXX: This is mainly used by the DNS director to fetch the actual backend
  * XXX: so it can compare DNS lookups with the actual IP.
  */
+
 struct backend *
 vdi_get_backend_if_simple(const struct director *d)
 {
@@ -499,48 +411,72 @@ vdi_get_backend_if_simple(const struct director *d)
 	if (vs2->magic != VDI_SIMPLE_MAGIC)
 		return NULL;
 	CAST_OBJ_NOTNULL(vs, d->priv, VDI_SIMPLE_MAGIC);
-	return vs->backend;
+	return (vs->backend);
 }
 
-static struct vbe_conn *
+/*--------------------------------------------------------------------
+ *
+ */
+
+void
+VBE_UseHealth(const struct director *vdi)
+{
+	struct vdi_simple *vs;
+
+	ASSERT_CLI();
+
+	if (strcmp(vdi->name, "simple"))
+		return;
+	CAST_OBJ_NOTNULL(vs, vdi->priv, VDI_SIMPLE_MAGIC);
+	if (vs->vrt->probe == NULL)
+		return;
+	VBP_Start(vs->backend, vs->vrt->probe, vs->vrt->hosthdr);
+}
+
+/*--------------------------------------------------------------------
+ *
+ */
+
+static struct vbc * __match_proto__(vdi_getfd_f)
 vdi_simple_getfd(const struct director *d, struct sess *sp)
 {
 	struct vdi_simple *vs;
-	struct vbe_conn *vc;
+	struct vbc *vc;
 
 	CHECK_OBJ_NOTNULL(sp, SESS_MAGIC);
 	CHECK_OBJ_NOTNULL(d, DIRECTOR_MAGIC);
 	CAST_OBJ_NOTNULL(vs, d->priv, VDI_SIMPLE_MAGIC);
-	vc = vbe_GetVbe(sp, vs->backend);
+	vc = vbe_GetVbe(sp, vs);
 	if (vc != NULL) {
 		FIND_TMO(first_byte_timeout,
-		    vc->first_byte_timeout, sp, vc->backend);
+		    vc->first_byte_timeout, sp, vs->vrt);
 		FIND_TMO(between_bytes_timeout,
-		    vc->between_bytes_timeout, sp, vc->backend);
+		    vc->between_bytes_timeout, sp, vs->vrt);
 	}
 	return (vc);
 }
 
 static unsigned
-vdi_simple_healthy(double now, const struct director *d, uintptr_t target)
+vdi_simple_healthy(const struct director *d, const struct sess *sp)
 {
 	struct vdi_simple *vs;
 
 	CHECK_OBJ_NOTNULL(d, DIRECTOR_MAGIC);
 	CAST_OBJ_NOTNULL(vs, d->priv, VDI_SIMPLE_MAGIC);
-	return (vbe_Healthy(now, target, vs->backend));
+	return (vbe_Healthy(vs, sp));
 }
 
-/*lint -e{818} not const-able */
 static void
-vdi_simple_fini(struct director *d)
+vdi_simple_fini(const struct director *d)
 {
 	struct vdi_simple *vs;
 
+	ASSERT_CLI();
 	CHECK_OBJ_NOTNULL(d, DIRECTOR_MAGIC);
 	CAST_OBJ_NOTNULL(vs, d->priv, VDI_SIMPLE_MAGIC);
 
-	VBE_DropRef(vs->backend);
+	VBP_Stop(vs->backend, vs->vrt->probe);
+	VBE_DropRefVcl(vs->backend);
 	free(vs->dir.vcl_name);
 	vs->dir.magic = 0;
 	FREE_OBJ(vs);
@@ -567,7 +503,11 @@ VRT_init_dir_simple(struct cli *cli, struct director **bp, int idx,
 	vs->dir.fini = vdi_simple_fini;
 	vs->dir.healthy = vdi_simple_healthy;
 
+	vs->vrt = t;
+
 	vs->backend = VBE_AddBackend(cli, t);
+	if (vs->backend->probe == NULL)
+		VBP_Start(vs->backend, vs->vrt->probe, vs->vrt->hosthdr);
 
 	bp[idx] = &vs->dir;
 }
