@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2009 Linpro AS
+ * Copyright (c) 2006-2011 Varnish Software AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -26,25 +26,15 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * We maintain a number of worker thread pools, to spread lock contention.
- *
- * Pools can be added on the fly, as a means to mitigate lock contention,
- * but can only be removed again by a restart. (XXX: we could fix that)
- *
- * Two threads herd the pools, one eliminates idle threads and aggregates
- * statistics for all the pools, the other thread creates new threads
- * on demand, subject to various numerical constraints.
- *
- * The algorithm for when to create threads needs to be reactive enough
- * to handle startup spikes, but sufficiently attenuated to not cause
- * thread pileups.  This remains subject for improvement.
+ * Write data to fd
+ * We try to use writev() if possible in order to minimize number of
+ * syscalls made and packets sent.  It also just might allow the worker
+ * thread to complete the request without holding stuff locked.
  */
 
 #include "config.h"
 
-#include "svnid.h"
-SVNID("$Id: cache_wrw.c 5043 2010-07-13 09:33:40Z tfheen $")
-
+#include <stdio.h>
 #include <sys/types.h>
 #include <sys/uio.h>
 
@@ -60,58 +50,92 @@ SVNID("$Id: cache_wrw.c 5043 2010-07-13 09:33:40Z tfheen $")
 #endif
 #endif /* SENDFILE_WORKS */
 
-#include "shmlog.h"
 #include "cache.h"
 
 /*--------------------------------------------------------------------
- * Write data to fd
- * We try to use writev() if possible in order to minimize number of
- * syscalls made and packets sent.  It also just might allow the worker
- * thread to complete the request without holding stuff locked.
  */
 
+int
+WRW_Error(const struct worker *w)
+{
+
+	return (w->wrw.werr);
+}
+
 void
 WRW_Reserve(struct worker *w, int *fd)
 {
+	struct wrw *wrw;
 
 	CHECK_OBJ_NOTNULL(w, WORKER_MAGIC);
-	AZ(w->wfd);
-	w->werr = 0;
-	w->liov = 0;
-	w->niov = 0;
-	w->wfd = fd;
+	wrw = &w->wrw;
+	AZ(wrw->wfd);
+	wrw->werr = 0;
+	wrw->liov = 0;
+	wrw->niov = 0;
+	wrw->ciov = wrw->siov;
+	wrw->wfd = fd;
 }
 
 static void
 WRW_Release(struct worker *w)
 {
+	struct wrw *wrw;
 
 	CHECK_OBJ_NOTNULL(w, WORKER_MAGIC);
-	w->werr = 0;
-	w->liov = 0;
-	w->niov = 0;
-	w->wfd = NULL;
+	wrw = &w->wrw;
+	AN(wrw->wfd);
+	wrw->werr = 0;
+	wrw->liov = 0;
+	wrw->niov = 0;
+	wrw->ciov = wrw->siov;
+	wrw->wfd = NULL;
 }
 
 unsigned
 WRW_Flush(struct worker *w)
 {
 	ssize_t i;
+	struct wrw *wrw;
+	char cbuf[32];
 
 	CHECK_OBJ_NOTNULL(w, WORKER_MAGIC);
-	AN(w->wfd);
-	if (*w->wfd >= 0 && w->niov > 0 && w->werr == 0) {
-		i = writev(*w->wfd, w->iov, w->niov);
-		if (i != w->liov) {
-			w->werr++;
-			WSL(w, SLT_Debug, *w->wfd,
+	wrw = &w->wrw;
+	AN(wrw->wfd);
+
+	/* For chunked, there must be one slot reserved for the chunked tail */
+	if (wrw->ciov < wrw->siov)
+		assert(wrw->niov < wrw->siov);
+
+	if (*wrw->wfd >= 0 && wrw->liov > 0 && wrw->werr == 0) {
+		if (wrw->ciov < wrw->siov && wrw->cliov > 0) {
+			bprintf(cbuf, "00%jx\r\n", (intmax_t)wrw->cliov);
+			i = strlen(cbuf);
+			wrw->iov[wrw->ciov].iov_base = cbuf;
+			wrw->iov[wrw->ciov].iov_len = i;
+			wrw->liov += i;
+
+			wrw->iov[wrw->niov].iov_base = cbuf + i - 2;
+			wrw->iov[wrw->niov++].iov_len = 2;
+			wrw->liov += 2;
+		} else if (wrw->ciov < wrw->siov) {
+			wrw->iov[wrw->ciov].iov_base = cbuf;
+			wrw->iov[wrw->ciov].iov_len = 0;
+		}
+		i = writev(*wrw->wfd, wrw->iov, wrw->niov);
+		if (i != wrw->liov) {
+			wrw->werr++;
+			WSL(w, SLT_Debug, *wrw->wfd,
 			    "Write error, retval = %d, len = %d, errno = %s",
-			    i, w->liov, strerror(errno));
+			    i, wrw->liov, strerror(errno));
 		}
 	}
-	w->liov = 0;
-	w->niov = 0;
-	return (w->werr);
+	wrw->liov = 0;
+	wrw->cliov = 0;
+	wrw->niov = 0;
+	if (wrw->ciov < wrw->siov)
+		wrw->ciov = wrw->niov++;
+	return (wrw->werr);
 }
 
 unsigned
@@ -120,7 +144,7 @@ WRW_FlushRelease(struct worker *w)
 	unsigned u;
 
 	CHECK_OBJ_NOTNULL(w, WORKER_MAGIC);
-	AN(w->wfd);
+	AN(w->wrw.wfd);
 	u = WRW_Flush(w);
 	WRW_Release(w);
 	return (u);
@@ -132,7 +156,7 @@ WRW_WriteH(struct worker *w, const txt *hh, const char *suf)
 	unsigned u;
 
 	CHECK_OBJ_NOTNULL(w, WORKER_MAGIC);
-	AN(w->wfd);
+	AN(w->wrw.wfd);
 	AN(w);
 	AN(hh);
 	AN(hh->b);
@@ -146,29 +170,79 @@ WRW_WriteH(struct worker *w, const txt *hh, const char *suf)
 unsigned
 WRW_Write(struct worker *w, const void *ptr, int len)
 {
+	struct wrw *wrw;
 
 	CHECK_OBJ_NOTNULL(w, WORKER_MAGIC);
-	AN(w->wfd);
-	if (len == 0 || *w->wfd < 0)
+	wrw = &w->wrw;
+	AN(wrw->wfd);
+	if (len == 0 || *wrw->wfd < 0)
 		return (0);
 	if (len == -1)
 		len = strlen(ptr);
-	if (w->niov == w->siov)
+	if (wrw->niov == wrw->siov + (wrw->ciov < wrw->siov ? 1 : 0))
 		(void)WRW_Flush(w);
-	w->iov[w->niov].iov_base = TRUST_ME(ptr);
-	w->iov[w->niov].iov_len = len;
-	w->liov += len;
-	w->niov++;
+	wrw->iov[wrw->niov].iov_base = TRUST_ME(ptr);
+	wrw->iov[wrw->niov].iov_len = len;
+	wrw->liov += len;
+	if (wrw->ciov < wrw->siov)
+		wrw->cliov += len;
+	wrw->niov++;
 	return (len);
 }
 
+void
+WRW_Chunked(struct worker *w)
+{
+	struct wrw *wrw;
+
+	CHECK_OBJ_NOTNULL(w, WORKER_MAGIC);
+	wrw = &w->wrw;
+
+	assert(wrw->ciov == wrw->siov);
+	/*
+	 * If there are not space for chunked header, a chunk of data and
+	 * a chunk tail, we might as well flush right away.
+	 */
+	if (wrw->niov + 3 >= wrw->siov)
+		(void)WRW_Flush(w);
+	wrw->ciov = wrw->niov++;
+	wrw->cliov = 0;
+	assert(wrw->ciov < wrw->siov);
+}
+
+/*
+ * XXX: It is not worth the complexity to attempt to get the
+ * XXX: end of chunk into the WRW_Flush(), because most of the time
+ * XXX: if not always, that is a no-op anyway, because the calling
+ * XXX: code already called WRW_Flush() to release local storage.
+ */
+
+void
+WRW_EndChunk(struct worker *w)
+{
+	struct wrw *wrw;
+
+	CHECK_OBJ_NOTNULL(w, WORKER_MAGIC);
+	wrw = &w->wrw;
+
+	assert(wrw->ciov < wrw->siov);
+	(void)WRW_Flush(w);
+	wrw->ciov = wrw->siov;
+	wrw->niov = 0;
+	wrw->cliov = 0;
+	(void)WRW_Write(w, "0\r\n\r\n", -1);
+}
+
+
 #ifdef SENDFILE_WORKS
 void
 WRW_Sendfile(struct worker *w, int fd, off_t off, unsigned len)
 {
+	struct wrw *wrw;
 
 	CHECK_OBJ_NOTNULL(w, WORKER_MAGIC);
-	AN(w->wfd);
+	wrw = &w->wrw;
+	AN(wrw->wfd);
 	assert(fd >= 0);
 	assert(len > 0);
 
@@ -176,31 +250,31 @@ WRW_Sendfile(struct worker *w, int fd, off_t off, unsigned len)
 	do {
 		struct sf_hdtr sfh;
 		memset(&sfh, 0, sizeof sfh);
-		if (w->niov > 0) {
-			sfh.headers = w->iov;
-			sfh.hdr_cnt = w->niov;
+		if (wrw->niov > 0) {
+			sfh.headers = wrw->iov;
+			sfh.hdr_cnt = wrw->niov;
 		}
-		if (sendfile(fd, *w->wfd, off, len, &sfh, NULL, 0) != 0)
-			w->werr++;
-		w->liov = 0;
-		w->niov = 0;
+		if (sendfile(fd, *wrw->wfd, off, len, &sfh, NULL, 0) != 0)
+			wrw->werr++;
+		wrw->liov = 0;
+		wrw->niov = 0;
 	} while (0);
 #elif defined(__linux__)
 	do {
 		if (WRK_Flush(w) == 0 &&
-		    sendfile(*w->wfd, fd, &off, len) != len)
-			w->werr++;
+		    sendfile(*wrw->wfd, fd, &off, len) != len)
+			wrw->werr++;
 	} while (0);
 #elif defined(__sun) && defined(HAVE_SENDFILEV)
 	do {
 		sendfilevec_t svvec[params->http_headers * 2 + 1];
 		size_t xferred = 0, expected = 0;
 		int i;
-		for (i = 0; i < w->niov; i++) {
+		for (i = 0; i < wrw->niov; i++) {
 			svvec[i].sfv_fd = SFV_FD_SELF;
 			svvec[i].sfv_flag = 0;
-			svvec[i].sfv_off = (off_t) w->iov[i].iov_base;
-			svvec[i].sfv_len = w->iov[i].iov_len;
+			svvec[i].sfv_off = (off_t) wrw->iov[i].iov_base;
+			svvec[i].sfv_len = wrw->iov[i].iov_len;
 			expected += svvec[i].sfv_len;
 		}
 		svvec[i].sfv_fd = fd;
@@ -208,17 +282,17 @@ WRW_Sendfile(struct worker *w, int fd, off_t off, unsigned len)
 		svvec[i].sfv_off = off;
 		svvec[i].sfv_len = len;
 		expected += svvec[i].sfv_len;
-		if (sendfilev(*w->wfd, svvec, i, &xferred) == -1 ||
+		if (sendfilev(*wrw->wfd, svvec, i, &xferred) == -1 ||
 		    xferred != expected)
-			w->werr++;
-		w->liov = 0;
-		w->niov = 0;
+			wrw->werr++;
+		wrw->liov = 0;
+		wrw->niov = 0;
 	} while (0);
 #elif defined(__sun) && defined(HAVE_SENDFILE)
 	do {
 		if (WRK_Flush(w) == 0 &&
-		    sendfile(*w->wfd, fd, &off, len) != len)
-			w->werr++;
+		    sendfile(*wrw->wfd, fd, &off, len) != len)
+			wrw->werr++;
 	} while (0);
 #else
 #error Unknown sendfile() implementation
