@@ -1,6 +1,6 @@
 /*-
  * Copyright (c) 2006 Verdens Gang AS
- * Copyright (c) 2006-2009 Linpro AS
+ * Copyright (c) 2006-2011 Varnish Software AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -31,9 +31,6 @@
 
 #include "config.h"
 
-#include "svnid.h"
-SVNID("$Id: varnishlog.c 5345 2010-09-29 11:35:47Z tfheen $")
-
 #include <errno.h>
 #include <fcntl.h>
 #include <signal.h>
@@ -41,7 +38,6 @@ SVNID("$Id: varnishlog.c 5345 2010-09-29 11:35:47Z tfheen $")
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
-#include <limits.h>
 
 #include "compat/daemon.h"
 
@@ -49,108 +45,88 @@ SVNID("$Id: varnishlog.c 5345 2010-09-29 11:35:47Z tfheen $")
 #include "vpf.h"
 
 #include "libvarnish.h"
-#include "shmlog.h"
+#include "vsl.h"
 #include "vre.h"
 #include "varnishapi.h"
 
 static int	b_flag, c_flag;
 
-/* -------------------------------------------------------------------*/
-
-static int
-name2tag(const char *n)
-{
-	int i;
-
-	for (i = 0; i < 256; i++) {
-		if (VSL_tags[i] == NULL)
-			continue;
-		if (!strcasecmp(n, VSL_tags[i]))
-			return (i);
-	}
-	return (-1);
-}
-
 /* Ordering-----------------------------------------------------------*/
 
 static struct vsb	*ob[65536];
 static unsigned char	flg[65536];
-static enum shmlogtag   last[65536];
+static enum VSL_tag_e   last[65536];
+static uint64_t       bitmap[65536];
 #define F_INVCL		(1 << 0)
-#define F_MATCH		(1 << 1)
-
-static int		match_tag = -1;
-static vre_t		*match_re;
 
 static void
-h_order_finish(int fd)
+h_order_finish(int fd, struct VSM_data *vd)
 {
 
-	vsb_finish(ob[fd]);
-	if (vsb_len(ob[fd]) > 1 &&
-	    (match_tag == -1 || flg[fd] & F_MATCH))
-		printf("%s\n", vsb_data(ob[fd]));
-	flg[fd] &= ~F_MATCH;
-	vsb_clear(ob[fd]);
+	AZ(VSB_finish(ob[fd]));
+	if (VSB_len(ob[fd]) > 1 && VSL_Matched(vd, bitmap[fd])) {
+		printf("%s", VSB_data(ob[fd]));
+	}
+	bitmap[fd] = 0;
+	VSB_clear(ob[fd]);
 }
 
 static void
-clean_order(void)
+clean_order(struct VSM_data *vd)
 {
 	unsigned u;
 
 	for (u = 0; u < 65536; u++) {
 		if (ob[u] == NULL)
 			continue;
-		vsb_finish(ob[u]);
-		if (vsb_len(ob[u]) > 1 &&
-		    (match_tag == -1 || flg[u] & F_MATCH))
-			printf("%s\n", vsb_data(ob[u]));
+		AZ(VSB_finish(ob[u]));
+		if (VSB_len(ob[u]) > 1 && VSL_Matched(vd, bitmap[u])) {
+			printf("%s\n", VSB_data(ob[u]));
+		}
 		flg[u] = 0;
-		vsb_clear(ob[u]);
+		bitmap[u] = 0;
+		VSB_clear(ob[u]);
 	}
 }
 
 static int
-h_order(void *priv, enum shmlogtag tag, unsigned fd, unsigned len,
-    unsigned spec, const char *ptr)
+h_order(void *priv, enum VSL_tag_e tag, unsigned fd, unsigned len,
+    unsigned spec, const char *ptr, uint64_t bm)
 {
 	char type;
 
-	(void)priv;
+	struct VSM_data *vd = priv;
+
+	bitmap[fd] |= bm;
 
 	type = (spec & VSL_S_CLIENT) ? 'c' :
 	    (spec & VSL_S_BACKEND) ? 'b' : '-';
 
 	if (!(spec & (VSL_S_CLIENT|VSL_S_BACKEND))) {
 		if (!b_flag && !c_flag)
-			VSL_H_Print(stdout, tag, fd, len, spec, ptr);
+			(void)VSL_H_Print(stdout, tag, fd, len, spec, ptr, bm);
 		return (0);
 	}
 	if (ob[fd] == NULL) {
-		ob[fd] = vsb_newauto();
+		ob[fd] = VSB_new_auto();
 		assert(ob[fd] != NULL);
 	}
-	if (tag == match_tag &&
-	    VRE_exec(match_re, ptr, len, 0, 0, NULL, 0) > 0)
-		flg[fd] |= F_MATCH;
-
 	if ((tag == SLT_BackendOpen || tag == SLT_SessionOpen ||
 		(tag == SLT_ReqStart &&
 		    last[fd] != SLT_SessionOpen &&
 		    last[fd] != SLT_VCL_acl) ||
 		(tag == SLT_BackendXID &&
 		    last[fd] != SLT_BackendOpen)) &&
-	    vsb_len(ob[fd]) != 0) {
+	    VSB_len(ob[fd]) != 0) {
 		/*
 		 * This is the start of a new request, yet we haven't seen
 		 * the end of the previous one.  Spit it out anyway before
 		 * starting on the new one.
 		 */
 		if (last[fd] != SLT_SessionClose)
-			vsb_printf(ob[fd], "%5d %-12s %c %s\n",
+			VSB_printf(ob[fd], "%5d %-12s %c %s\n",
 			    fd, "Interrupted", type, VSL_tags[tag]);
-		h_order_finish(fd);
+		h_order_finish(fd, vd);
 	}
 
 	last[fd] = tag;
@@ -158,17 +134,17 @@ h_order(void *priv, enum shmlogtag tag, unsigned fd, unsigned len,
 	switch (tag) {
 	case SLT_VCL_call:
 		if (flg[fd] & F_INVCL)
-			vsb_cat(ob[fd], "\n");
+			VSB_cat(ob[fd], "\n");
 		else
 			flg[fd] |= F_INVCL;
-		vsb_printf(ob[fd], "%5d %-12s %c %.*s",
+		VSB_printf(ob[fd], "%5d %-12s %c %.*s",
 		    fd, VSL_tags[tag], type, len, ptr);
 		return (0);
 	case SLT_VCL_trace:
 	case SLT_VCL_return:
 		if (flg[fd] & F_INVCL) {
-			vsb_cat(ob[fd], " ");
-			vsb_bcat(ob[fd], ptr, len);
+			VSB_cat(ob[fd], " ");
+			VSB_bcat(ob[fd], ptr, len);
 			return (0);
 		}
 		break;
@@ -176,17 +152,17 @@ h_order(void *priv, enum shmlogtag tag, unsigned fd, unsigned len,
 		break;
 	}
 	if (flg[fd] & F_INVCL) {
-		vsb_cat(ob[fd], "\n");
+		VSB_cat(ob[fd], "\n");
 		flg[fd] &= ~F_INVCL;
 	}
-	vsb_printf(ob[fd], "%5d %-12s %c %.*s\n",
+	VSB_printf(ob[fd], "%5d %-12s %c %.*s\n",
 	    fd, VSL_tags[tag], type, len, ptr);
 	switch (tag) {
 	case SLT_ReqEnd:
 	case SLT_BackendClose:
 	case SLT_BackendReuse:
 	case SLT_StatSess:
-		h_order_finish(fd);
+		h_order_finish(fd, vd);
 		break;
 	default:
 		break;
@@ -195,24 +171,10 @@ h_order(void *priv, enum shmlogtag tag, unsigned fd, unsigned len,
 }
 
 static void
-do_order(struct VSL_data *vd, int argc, char **argv)
+do_order(struct VSM_data *vd)
 {
 	int i;
-	const char *error;
-	int erroroffset;
 
-	if (argc == 2) {
-		match_tag = name2tag(argv[0]);
-		if (match_tag < 0) {
-			fprintf(stderr, "Tag \"%s\" unknown\n", argv[0]);
-			exit(2);
-		}
-		match_re = VRE_compile(argv[1], 0, &error, &erroroffset);
-		if (match_re == NULL) {
-			fprintf(stderr, "Invalid regex: %s\n", error);
-			exit(2);
-		}
-	}
 	if (!b_flag) {
 		VSL_Select(vd, SLT_SessionOpen);
 		VSL_Select(vd, SLT_SessionClose);
@@ -224,15 +186,15 @@ do_order(struct VSL_data *vd, int argc, char **argv)
 		VSL_Select(vd, SLT_BackendReuse);
 	}
 	while (1) {
-		i = VSL_Dispatch(vd, h_order, NULL);
+		i = VSL_Dispatch(vd, h_order, vd);
 		if (i == 0) {
-			clean_order();
-			fflush(stdout);
+			clean_order(vd);
+			AZ(fflush(stdout));
 		}
 		else if (i < 0)
 			break;
 	}
-	clean_order();
+	clean_order(vd);
 }
 
 /*--------------------------------------------------------------------*/
@@ -268,28 +230,30 @@ open_log(const char *w_arg, int a_flag)
 }
 
 static void
-do_write(struct VSL_data *vd, const char *w_arg, int a_flag)
+do_write(const struct VSM_data *vd, const char *w_arg, int a_flag)
 {
 	int fd, i, l;
-	unsigned char *p;
+	uint32_t *p;
 
 	fd = open_log(w_arg, a_flag);
-	signal(SIGHUP, sighup);
+	XXXAN(fd >= 0);
+	(void)signal(SIGHUP, sighup);
 	while (1) {
-		i = VSL_NextLog(vd, &p);
+		i = VSL_NextLog(vd, &p, NULL);
 		if (i < 0)
 			break;
 		if (i > 0) {
-			l = SHMLOG_LEN(p);
-			i = write(fd, p, SHMLOG_NEXTTAG + l);
+			l = VSL_LEN(p);
+			i = write(fd, p, 8L + VSL_WORDS(l) * 4L);
 			if (i < 0) {
 				perror(w_arg);
 				exit(1);
 			}
 		}
 		if (reopen) {
-			close(fd);
+			AZ(close(fd));
 			fd = open_log(w_arg, a_flag);
+			XXXAN(fd >= 0);
 			reopen = 0;
 		}
 	}
@@ -302,44 +266,43 @@ static void
 usage(void)
 {
 	fprintf(stderr, "usage: varnishlog "
-	    "%s [-aDoV] [-n varnish_name] [-P file] [-w file]\n", VSL_USAGE);
+	    "%s [-aDV] [-o [tag regex]] [-n varnish_name] [-P file] [-w file]\n", VSL_USAGE);
 	exit(1);
 }
 
 int
-main(int argc, char **argv)
+main(int argc, char * const *argv)
 {
 	int c;
-	int a_flag = 0, D_flag = 0, o_flag = 0, u_flag = 0;
-	const char *n_arg = NULL;
+	int a_flag = 0, D_flag = 0, O_flag = 0, u_flag = 0, m_flag = 0;
 	const char *P_arg = NULL;
 	const char *w_arg = NULL;
-	struct pidfh *pfh = NULL;
-	struct VSL_data *vd;
+	struct vpf_fh *pfh = NULL;
+	struct VSM_data *vd;
 
-	vd = VSL_New();
+	vd = VSM_New();
+	VSL_Setup(vd);
 
-	while ((c = getopt(argc, argv, VSL_ARGS "aDon:P:uVw:")) != -1) {
+	while ((c = getopt(argc, argv, VSL_ARGS "aDP:uVw:oO")) != -1) {
 		switch (c) {
 		case 'a':
 			a_flag = 1;
 			break;
 		case 'b':
 			b_flag = 1;
-			VSL_Arg(vd, c, optarg);
+			AN(VSL_Arg(vd, c, optarg));
 			break;
 		case 'c':
 			c_flag = 1;
-			VSL_Arg(vd, c, optarg);
+			AN(VSL_Arg(vd, c, optarg));
 			break;
 		case 'D':
 			D_flag = 1;
 			break;
-		case 'n':
-			n_arg = optarg;
+		case 'o': /* ignored for compatibility with older versions */
 			break;
-		case 'o':
-			o_flag = 1;
+		case 'O':
+			O_flag = 1;
 			break;
 		case 'P':
 			P_arg = optarg;
@@ -348,11 +311,13 @@ main(int argc, char **argv)
 			u_flag = 1;
 			break;
 		case 'V':
-			varnish_version("varnishlog");
+			VCS_Message("varnishlog");
 			exit(0);
 		case 'w':
 			w_arg = optarg;
 			break;
+		case 'm':
+			m_flag = 1; /* fall through */
 		default:
 			if (VSL_Arg(vd, c, optarg) > 0)
 				break;
@@ -360,13 +325,16 @@ main(int argc, char **argv)
 		}
 	}
 
-	if (o_flag && w_arg != NULL)
+	if (O_flag && m_flag)
 		usage();
 
-	if (VSL_OpenLog(vd, n_arg))
+	if ((argc - optind) > 0)
+		usage();
+
+	if (VSL_Open(vd, 1))
 		exit(1);
 
-	if (P_arg && (pfh = vpf_open(P_arg, 0644, NULL)) == NULL) {
+	if (P_arg && (pfh = VPF_Open(P_arg, 0644, NULL)) == NULL) {
 		perror(P_arg);
 		exit(1);
 	}
@@ -374,12 +342,12 @@ main(int argc, char **argv)
 	if (D_flag && varnish_daemon(0, 0) == -1) {
 		perror("daemon()");
 		if (pfh != NULL)
-			vpf_remove(pfh);
+			VPF_Remove(pfh);
 		exit(1);
 	}
 
 	if (pfh != NULL)
-		vpf_write(pfh);
+		VPF_Write(pfh);
 
 	if (w_arg != NULL)
 		do_write(vd, w_arg, a_flag);
@@ -387,8 +355,8 @@ main(int argc, char **argv)
 	if (u_flag)
 		setbuf(stdout, NULL);
 
-	if (o_flag)
-		do_order(vd, argc - optind, argv + optind);
+	if (!O_flag)
+		do_order(vd);
 
 	while (VSL_Dispatch(vd, VSL_H_Print, stdout) >= 0) {
 		if (fflush(stdout) != 0) {
@@ -398,6 +366,6 @@ main(int argc, char **argv)
 	}
 
 	if (pfh != NULL)
-		vpf_remove(pfh);
+		VPF_Remove(pfh);
 	exit(0);
 }
