@@ -1,5 +1,5 @@
-/*
- * Copyright (c) 2008-2009 Linpro AS
+/*-
+ * Copyright (c) 2008-2011 Varnish Software AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -25,10 +25,15 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: vtc.h 4510 2010-01-29 09:56:01Z phk $
  */
 
+#include <limits.h>
+#include <signal.h>
 #include <pthread.h>
+#ifdef HAVE_PTHREAD_NP_H
+#include <pthread_np.h>
+#endif
+#include "vqueue.h"
 
 struct vsb;
 struct vtclog;
@@ -47,18 +52,15 @@ struct cmds {
 void parse_string(char *buf, const struct cmds *cmd, void *priv,
     struct vtclog *vl);
 
-cmd_f cmd_dump;
 cmd_f cmd_delay;
 cmd_f cmd_server;
 cmd_f cmd_client;
 cmd_f cmd_varnish;
 cmd_f cmd_sema;
 
-extern int vtc_verbosity;
-extern int vtc_error;		/* Error, bail out */
+extern volatile sig_atomic_t vtc_error; /* Error, bail out */
 extern int vtc_stop;		/* Abandon current test, no error */
 extern pthread_t	vtc_thread;
-extern char *vtc_tmpdir;
 
 void init_sema(void);
 
@@ -66,15 +68,21 @@ void http_process(struct vtclog *vl, const char *spec, int sock, int sfd);
 
 void cmd_server_genvcl(struct vsb *vsb);
 
-void vtc_loginit(void);
-void vtc_logreset(void);
-const char *vtc_logfull(void);
+void vtc_loginit(char *buf, unsigned buflen);
 struct vtclog *vtc_logopen(const char *id);
 void vtc_logclose(struct vtclog *vl);
 void vtc_log(struct vtclog *vl, unsigned lvl, const char *fmt, ...);
 void vtc_dump(struct vtclog *vl, unsigned lvl, const char *pfx,
-    const char *str);
+    const char *str, int len);
+void vtc_hexdump(struct vtclog *vl, unsigned lvl, const char *pfx,
+    const unsigned char *str, int len);
+
+int exec_file(const char *fn, const char *script, const char *tmpdir,
+    char *logbuf, unsigned loglen);
 
 void macro_def(struct vtclog *vl, const char *instance, const char *name,
     const char *fmt, ...);
-struct vsb *macro_expand(const char *name);
+struct vsb *macro_expand(struct vtclog *vl, const char *text);
+
+void extmacro_def(const char *name, const char *fmt, ...);
+const char *extmacro_get(const char *name);
