@@ -1,5 +1,5 @@
-/*
- * Copyright (c) 2008-2010 Redpill Linpro AS
+/*-
+ * Copyright (c) 2008-2011 Varnish Software AS
  * All rights reserved.
  *
  * Author: Poul-Henning Kamp <phk@phk.freebsd.dk>
@@ -28,27 +28,23 @@
 
 #include "config.h"
 
-#include "svnid.h"
-SVNID("$Id: vtc_log.c 4578 2010-02-18 11:48:32Z phk $")
-
 #include <stdio.h>
 #include <string.h>
 #include <fcntl.h>
 #include <stdlib.h>
 #include <unistd.h>
-#include <pthread.h>
 #include <stdarg.h>
 
 #include "libvarnish.h"
 #include "vsb.h"
 #include "miniobj.h"
+#include "vas.h"
 
 #include "vtc.h"
 
-int vtc_verbosity = 0;
-
-static struct vsb	*vtclog_full;
 static pthread_mutex_t	vtclog_mtx;
+static char		*vtclog_buf;
+static unsigned		vtclog_left;
 
 struct vtclog {
 	unsigned	magic;
@@ -58,30 +54,20 @@ struct vtclog {
 	pthread_mutex_t	mtx;
 };
 
+static pthread_key_t log_key;
+static double t0;
+
 /**********************************************************************/
 
 void
-vtc_loginit()
+vtc_loginit(char *buf, unsigned buflen)
 {
 
-	vtclog_full = vsb_newauto();
-	AN(vtclog_full);
+	t0 = TIM_mono();
+	vtclog_buf = buf;
+	vtclog_left = buflen;
 	AZ(pthread_mutex_init(&vtclog_mtx, NULL));
-}
-
-void
-vtc_logreset()
-{
-
-	vsb_clear(vtclog_full);
-}
-
-const char *
-vtc_logfull(void)
-{
-	vsb_finish(vtclog_full);
-	AZ(vsb_overflowed(vtclog_full));
-	return (vsb_data(vtclog_full));
+	AZ(pthread_key_create(&log_key, NULL));
 }
 
 /**********************************************************************/
@@ -95,8 +81,9 @@ vtc_logopen(const char *id)
 	ALLOC_OBJ(vl, VTCLOG_MAGIC);
 	AN(vl);
 	vl->id = id;
-	vl->vsb = vsb_newauto();
+	vl->vsb = VSB_new_auto();
 	AZ(pthread_mutex_init(&vl->mtx, NULL));
+	AZ(pthread_setspecific(log_key, vl));
 	return (vl);
 }
 
@@ -105,55 +92,60 @@ vtc_logclose(struct vtclog *vl)
 {
 
 	CHECK_OBJ_NOTNULL(vl, VTCLOG_MAGIC);
-	vsb_delete(vl->vsb);
+	VSB_delete(vl->vsb);
 	AZ(pthread_mutex_destroy(&vl->mtx));
 	FREE_OBJ(vl);
 }
 
 static const char * const lead[] = {
 	"----",
-	"#   ",
-	"##  ",
-	"### ",
-	"####"
+	"*   ",
+	"**  ",
+	"*** ",
+	"****"
 };
 
 #define NLEAD (sizeof(lead)/sizeof(lead[0]))
 
 static void
-vtc_log_emit(struct vtclog *vl, unsigned lvl)
+vtc_log_emit(const struct vtclog *vl, unsigned lvl)
 {
+	int l;
+
 	if (vtc_stop && lvl == 0)
 		return;
+	l = VSB_len(vl->vsb);
 	AZ(pthread_mutex_lock(&vtclog_mtx));
-	vsb_cat(vtclog_full, vsb_data(vl->vsb));
+	assert(vtclog_left > l);
+	memcpy(vtclog_buf,VSB_data(vl->vsb), l);
+	vtclog_buf += l;
+	*vtclog_buf = '\0';
+	vtclog_left -= l;
 	AZ(pthread_mutex_unlock(&vtclog_mtx));
-
-	if (lvl <= vtc_verbosity)
-		(void)fputs(vsb_data(vl->vsb), stdout);
 }
 
 //lint -e{818}
 void
 vtc_log(struct vtclog *vl, unsigned lvl, const char *fmt, ...)
 {
+	double tx;
 
 	CHECK_OBJ_NOTNULL(vl, VTCLOG_MAGIC);
+	tx = TIM_mono() - t0;
 	AZ(pthread_mutex_lock(&vl->mtx));
 	assert(lvl < NLEAD);
-	vsb_clear(vl->vsb);
-	vsb_printf(vl->vsb, "%s %-4s ", lead[lvl], vl->id);
+	VSB_clear(vl->vsb);
+	VSB_printf(vl->vsb, "%s %-4s %4.1f ", lead[lvl], vl->id, tx);
 	va_list ap;
 	va_start(ap, fmt);
-	(void)vsb_vprintf(vl->vsb, fmt, ap);
+	(void)VSB_vprintf(vl->vsb, fmt, ap);
 	va_end(ap);
-	vsb_putc(vl->vsb, '\n');
-	vsb_finish(vl->vsb);
-	AZ(vsb_overflowed(vl->vsb));
+	VSB_putc(vl->vsb, '\n');
+	AZ(VSB_finish(vl->vsb));
 
 	vtc_log_emit(vl, lvl);
 
-	vsb_clear(vl->vsb);
+	VSB_clear(vl->vsb);
 	AZ(pthread_mutex_unlock(&vl->mtx));
 	if (lvl == 0) {
 		vtc_error = 1;
@@ -168,46 +160,55 @@ vtc_log(struct vtclog *vl, unsigned lvl, const char *fmt, ...)
 
 //lint -e{818}
 void
-vtc_dump(struct vtclog *vl, unsigned lvl, const char *pfx, const char *str)
+vtc_dump(struct vtclog *vl, unsigned lvl, const char *pfx, const char *str, int len)
 {
 	int nl = 1;
+	unsigned l;
+	double tx;
 
 	CHECK_OBJ_NOTNULL(vl, VTCLOG_MAGIC);
+	tx = TIM_mono() - t0;
 	assert(lvl < NLEAD);
 	AZ(pthread_mutex_lock(&vl->mtx));
-	vsb_clear(vl->vsb);
+	VSB_clear(vl->vsb);
 	if (pfx == NULL)
 		pfx = "";
 	if (str == NULL)
-		vsb_printf(vl->vsb, "%s %-4s %s(null)\n",
-		    lead[lvl], vl->id, pfx);
-	else
-		for(; *str != '\0'; str++) {
+		VSB_printf(vl->vsb, "%s %-4s %4.1f %s(null)\n",
+		    lead[lvl], vl->id, tx, pfx);
+	else {
+		if (len == -1)
+			len = strlen(str);
+		for (l = 0; l < len; l++, str++) {
+			if (l > 512) {
+				VSB_printf(vl->vsb, "...");
+				break;
+			}
 			if (nl) {
-				vsb_printf(vl->vsb, "%s %-4s %s| ",
-				    lead[lvl], vl->id, pfx);
+				VSB_printf(vl->vsb, "%s %-4s %4.1f %s| ",
+				    lead[lvl], vl->id, tx, pfx);
 				nl = 0;
 			}
 			if (*str == '\r')
-				vsb_printf(vl->vsb, "\\r");
+				VSB_printf(vl->vsb, "\\r");
 			else if (*str == '\t')
-				vsb_printf(vl->vsb, "\\t");
+				VSB_printf(vl->vsb, "\\t");
 			else if (*str == '\n') {
-				vsb_printf(vl->vsb, "\\n\n");
+				VSB_printf(vl->vsb, "\\n\n");
 				nl = 1;
 			} else if (*str < 0x20 || *str > 0x7e)
-				vsb_printf(vl->vsb, "\\x%02x", *str);
+				VSB_printf(vl->vsb, "\\x%02x", (*str) & 0xff);
 			else
-				vsb_printf(vl->vsb, "%c", *str);
+				VSB_printf(vl->vsb, "%c", *str);
 		}
+	}
 	if (!nl)
-		vsb_printf(vl->vsb, "\n");
-	vsb_finish(vl->vsb);
-	AZ(vsb_overflowed(vl->vsb));
+		VSB_printf(vl->vsb, "\n");
+	AZ(VSB_finish(vl->vsb));
 
 	vtc_log_emit(vl, lvl);
 
-	vsb_clear(vl->vsb);
+	VSB_clear(vl->vsb);
 	AZ(pthread_mutex_unlock(&vl->mtx));
 	if (lvl == 0) {
 		vtc_error = 1;
@@ -215,3 +216,81 @@ vtc_dump(struct vtclog *vl, unsigned lvl, const char *pfx, const char *str)
 			pthread_exit(NULL);
 	}
 }
+
+/**********************************************************************
+ * Hexdump
+ */
+
+//lint -e{818}
+void
+vtc_hexdump(struct vtclog *vl, unsigned lvl, const char *pfx, const unsigned char *str, int len)
+{
+	int nl = 1;
+	unsigned l;
+
+	CHECK_OBJ_NOTNULL(vl, VTCLOG_MAGIC);
+	assert(len >= 0);
+	assert(lvl < NLEAD);
+	AZ(pthread_mutex_lock(&vl->mtx));
+	VSB_clear(vl->vsb);
+	if (pfx == NULL)
+		pfx = "";
+	if (str == NULL)
+		VSB_printf(vl->vsb, "%s %-4s %s(null)\n",
+		    lead[lvl], vl->id, pfx);
+	else {
+		for (l = 0; l < len; l++, str++) {
+			if (l > 512) {
+				VSB_printf(vl->vsb, "...");
+				break;
+			}
+			if (nl) {
+				VSB_printf(vl->vsb, "%s %-4s %s| ",
+				    lead[lvl], vl->id, pfx);
+				nl = 0;
+			}
+			VSB_printf(vl->vsb, " %02x", *str);
+			if ((l & 0xf) == 0xf) {
+				VSB_printf(vl->vsb, "\n");
+				nl = 1;
+			}
+		}
+	}
+	if (!nl)
+		VSB_printf(vl->vsb, "\n");
+	AZ(VSB_finish(vl->vsb));
+
+	vtc_log_emit(vl, lvl);
+
+	VSB_clear(vl->vsb);
+	AZ(pthread_mutex_unlock(&vl->mtx));
+	if (lvl == 0) {
+		vtc_error = 1;
+		if (pthread_self() != vtc_thread)
+			pthread_exit(NULL);
+	}
+}
+
+/**********************************************************************/
+
+static void
+vtc_log_VAS_Fail(const char *func, const char *file, int line,
+    const char *cond, int err, int xxx)
+{
+	struct vtclog *vl;
+
+	(void)err;
+	(void)xxx;
+	vl = pthread_getspecific(log_key);
+	if (vl == NULL) {
+		fprintf(stderr,
+		    "Assert error in %s(), %s line %d:\n"
+		    "  Condition(%s) not true.\n",
+		    func, file, line, cond);
+	} else {
+		vtc_log(vl, 0, "Assert error in %s(), %s line %d:"
+		    "  Condition(%s) not true.\n", func, file, line, cond);
+	}
+}
+
+vas_f *VAS_Fail = vtc_log_VAS_Fail;
